@@ -517,7 +517,9 @@ retrieve_schema_data(const char *mod_name, const char *mod_rev, const char *subm
             match = u + 1;
         }
         if (!match) {
-            WRN("Session %u: unable to identify revision of the schema \"%s\" from the available server side information.",
+            /* valid situation if we are retrieving YANG 1.1 schema and have only capabilities for now
+             * (when loading ietf-datastore for ietf-yang-library) */
+            VRB("Session %u: unable to identify revision of the schema \"%s\" from the available server side information.",
                 clb_data->session->id, mod_name);
         }
     }
@@ -527,7 +529,7 @@ retrieve_schema_data(const char *mod_name, const char *mod_rev, const char *subm
             rev = sub_rev;
         } else if (match) {
             if (!clb_data->schemas[match - 1].submodules) {
-                WRN("Session %u: Unable to identify revision of the requested submodule \"%s\", in schema \"%s\", from the available server side information.",
+                VRB("Session %u: Unable to identify revision of the requested submodule \"%s\", in schema \"%s\", from the available server side information.",
                     clb_data->session->id, submod_name, mod_name);
             } else {
                 for (v = 0; clb_data->schemas[match - 1].submodules[v].name; ++v) {
@@ -547,7 +549,7 @@ retrieve_schema_data(const char *mod_name, const char *mod_rev, const char *subm
         rev = mod_rev;
     }
 
-    VRB("Session %u: retreiving data for schema \"%s\", revision \"%s\".", clb_data->session->id, name, rev);
+    VRB("Session %u: retreiving data for schema \"%s\", revision \"%s\".", clb_data->session->id, name, rev ? rev : "<latest>");
 
     if (match) {
         /* we have enough information to avoid communication with server and try to get
@@ -1257,7 +1259,7 @@ _non_blocking_connect(int timeout, int *sock_pending, struct addrinfo *res, stru
     char str[INET6_ADDRSTRLEN];
 
     if (sock_pending && *sock_pending != -1) {
-        VRB("Trying to connect the pending socket=%d.", *sock_pending );
+        VRB("Trying to connect the pending socket %d.", *sock_pending );
         sock = *sock_pending;
     } else {
         assert(res);
@@ -1358,7 +1360,7 @@ nc_sock_connect(const char *host, uint16_t port, int timeout, struct nc_keepaliv
 {
     int i, opt;
     int sock = sock_pending ? *sock_pending : -1;
-    struct addrinfo hints, *res_list, *res;
+    struct addrinfo hints, *res_list = NULL, *res;
     char *buf, port_s[6]; /* length of string representation of short int */
     void *addr;
 
@@ -1386,7 +1388,7 @@ nc_sock_connect(const char *host, uint16_t port, int timeout, struct nc_keepaliv
                     continue;
                 } else {
                     /* timeout, keep pending socket */
-                    return -1;
+                    break;
                 }
             }
             VRB("Successfully connected to %s:%s over %s.", host, port_s, (res->ai_family == AF_INET6) ? "IPv6" : "IPv4");
@@ -1429,6 +1431,9 @@ nc_sock_connect(const char *host, uint16_t port, int timeout, struct nc_keepaliv
     return sock;
 
 error:
+    if (res_list) {
+        freeaddrinfo(res_list);
+    }
     if (sock != -1) {
         close(sock);
     }
@@ -1493,12 +1498,6 @@ get_msg(struct nc_session *session, int timeout, uint64_t msgid, struct lyxml_el
 
     /* we read notif, want a rpc-reply */
     if (msgid && (msgtype == NC_MSG_NOTIF)) {
-        if (!ATOMIC_LOAD(session->opts.client.ntf_tid)) {
-            ERR("Session %u: received a <notification> but session is not subscribed.", session->id);
-            lyxml_free(session->ctx, xml);
-            return NC_MSG_ERROR;
-        }
-
         cont_ptr = &session->opts.client.notifs;
         while (*cont_ptr) {
             cont_ptr = &((*cont_ptr)->next);
