@@ -692,14 +692,17 @@ auth_password_get_pwd_hash(const char *username)
     }
 
     if (!strcmp(pwd->pw_passwd, "x")) {
-        #ifndef __QNXNTO__
-            getspnam_r(username, &spwd_buf, buf, 256, &spwd);
-        #else
-            spwd = getspnam_r(username, &spwd_buf, buf, 256);
-        #endif
+# ifndef __QNXNTO__
+        getspnam_r(username, &spwd_buf, buf, 256, &spwd);
+# else
+        spwd = getspnam_r(username, &spwd_buf, buf, 256);
+# endif
         if (!spwd) {
             VRB("Failed to retrieve the shadow entry for \"%s\".", username);
             return NULL;
+        } else if ((spwd->sp_expire > -1) && (spwd->sp_expire <= (time(NULL) / (60 * 60 * 24)))) {
+            WRN("User \"%s\" account has expired.", username);
+            return NULL;
         }
 
         pass_hash = spwd->sp_pwdp;
@@ -1130,11 +1133,11 @@ nc_sshcb_msg(ssh_session UNUSED(sshsession), ssh_message msg, void *data)
     }
 
     VRB("Received an SSH message \"%s\" of subtype \"%s\".", str_type, str_subtype);
-    if ((session->status == NC_STATUS_CLOSING) || (session->status == NC_STATUS_INVALID)) {
+    if (!session || (session->status == NC_STATUS_CLOSING) || (session->status == NC_STATUS_INVALID)) {
         /* "valid" situation if, for example, receiving some auth or channel request timeouted,
          * but we got it now, during session free */
         VRB("SSH message arrived on a %s session, the request will be denied.",
-            (session->status == NC_STATUS_CLOSING ? "closing" : "invalid"));
+            (session && session->status == NC_STATUS_CLOSING ? "closing" : "invalid"));
         ssh_message_reply_default(msg);
         return 0;
     }
@@ -1501,7 +1504,7 @@ nc_session_accept_ssh_channel(struct nc_session *orig_session, struct nc_session
     }
 
     /* assign new SID atomically */
-    new_session->id = ATOMIC_INC(&server_opts.new_session_id);
+    new_session->id = ATOMIC_INC(server_opts.new_session_id);
 
     /* NETCONF handshake */
     msgtype = nc_handshake_io(new_session);
@@ -1572,7 +1575,7 @@ nc_ps_accept_ssh_channel(struct nc_pollsession *ps, struct nc_session **session)
     }
 
     /* assign new SID atomically */
-    new_session->id = ATOMIC_INC(&server_opts.new_session_id);
+    new_session->id = ATOMIC_INC(server_opts.new_session_id);
 
     /* NETCONF handshake */
     msgtype = nc_handshake_io(new_session);
